@@ -6,7 +6,7 @@ from typing import Text
 from django.conf import settings
 from django.db import connection, OperationalError
 
-from isc_common.common import blinkString
+from isc_common.common import blinkString, uuid4
 
 logger = logging.getLogger(__name__)
 
@@ -35,7 +35,7 @@ def create_tmp_mat_view(sql_str, params=[], indexes=[], mat_view_name=None) -> T
     if mat_view_name != None:
         settings.LOCKS.acquire(key)
     else:
-        mat_view_name = f'tmp_{str(uuid.uuid4()).replace("-", "_")}'
+        mat_view_name = f'tmp_{uuid4()}'
 
     if isinstance(indexes, list):
         indexes = [f'''CREATE INDEX {mat_view_name}_{index}_idx ON {mat_view_name} USING btree ("{index}")''' for index in indexes]
@@ -81,6 +81,110 @@ def create_view(sql_str, params=[], view_name=None) -> None:
         settings.LOCKS.release(key)
 
 
+def create_insert_update_delete_function_of_table(
+        table_name,
+        table_schema='public',
+        func_params=[('id', 'bigint')],
+        pk_names=['id']
+):
+    columns_sql_str = f'''select column_name, data_type, ordinal_position
+                                from information_schema.columns
+                                    where table_schema = %s
+                                and table_name = %s
+                                order by ordinal_position'''
+
+    def insert_proc_body_str(fields, where_clause):
+        return f'''insert into {table_name}_tbl ({fields}) select {fields} from {table_name}_view where {where_clause}'''
+
+    with connection.cursor() as cursor:
+        cursor.execute(columns_sql_str, [table_schema, f'{table_name}_view'])
+        rows = cursor.fetchall()
+
+        rows_without_pk_field = [row for row in rows if row[0] not in pk_names]
+
+        def func_param_str(param_item):
+            return f'{param_item[0]} {param_item[1]}'
+
+        def func_where_clause_str(param_item, prefix=None):
+            return f'''{f'{prefix}.' if prefix else ''}"{param_item[0]}"=f{param_item[0]}'''
+
+        def func_where_clause_str1(param_item, prefix1=None, prefix2=None):
+            prefix2 = '' if prefix2 == None else f'{prefix2}.'
+            return f'''{f'{prefix1}.' if prefix1 else ''}"{param_item[0]}"={prefix2}"{param_item[0]}"'''
+
+        func_params1 = ','.join(f'f{func_param_str(a)}' for a in func_params)
+        func_params2 = ','.join(f'f{a[0]}' for a in func_params)
+        func_params_types = ','.join(f'{a[1]}' for a in func_params)
+
+        where_clause = ' and '.join(f'{func_where_clause_str(a)}' for a in func_params)
+        where_clause_prefix_s = ' and '.join(f'''{func_where_clause_str(param_item=a, prefix='s')}''' for a in func_params)
+        where_clause_prefix_tbl = ' and '.join(f'''{func_where_clause_str1(param_item=a, prefix1=f'{table_name}_tbl', prefix2='s')}''' for a in func_params)
+
+        fields = ','.join([f'"{row[0]}"' for row in rows])
+        fields_without_pk_field = ','.join([f'"{row[0]}"' for row in rows_without_pk_field])
+
+        fields_update = ','.join([f'"{row[0]}"=s."{row[0]}"' for row in rows_without_pk_field])
+
+        insert_func_name = f'insert_into_{table_name}'
+        insert_proc_str = f'''drop function if exists {insert_func_name}({func_params_types}) cascade;
+                                  create function {insert_func_name}({func_params1}) returns bigint
+                                        language plpgsql
+                                    as
+                                    $$
+                                    BEGIN
+                                        {insert_proc_body_str(fields=fields, where_clause=where_clause)};
+                                        return fid;
+                                    END;
+                                    $$;'''
+
+        cursor.execute(insert_proc_str)
+
+        update_proc_str = f'''drop function if exists update_{table_name}({func_params_types}) cascade;
+                            create function update_{table_name}({func_params1})  RETURNS bigint
+                                language plpgsql
+                            as
+                            $$
+                            declare
+                                _id bigint;
+                            BEGIN
+                            
+                                select a."id"
+                                into _id
+                                from {table_name}_tbl as a 
+                                where a."id" = fid;
+                            
+                                if _id is null then
+                                    return {insert_func_name}({func_params2});
+                                else
+                                    update {table_name}_tbl
+                                    set 
+                                       {fields_update}
+                                    from (select {fields}
+                                          from {table_name}_view) as s
+                                            where  {where_clause_prefix_tbl}
+                                            and {where_clause_prefix_s};
+                                            return fid;                                        
+                                end if;
+                               
+                            END;
+                            $$;'''
+
+        cursor.execute(update_proc_str)
+
+        delete_proc_str = f'''drop function if exists delete_{table_name}({func_params_types}) cascade;
+                                create function delete_{table_name}({func_params1}) returns void
+                                    language plpgsql
+                                as
+                                $$
+                                BEGIN
+                                    delete from {table_name}_tbl where {where_clause};
+                                END;
+                                $$;
+                                '''
+
+        cursor.execute(delete_proc_str)
+
+
 def create_table(fields_str=None, sql_str=None, temporary='', on_commit=None, params=[], indexes=[], unique_indexes=[], primary_key=None, table_name=None, drop=True) -> None:
     if table_name == None:
         raise Exception(f'Table must have name')
