@@ -32,7 +32,10 @@ class BodyWrapper(Wrapper):
 
 
 class Body:
-    def __init__(self, body):
+    session = None
+
+    def __init__(self, body, session):
+        self.session = session
         if not isinstance(body, dict):
             raise Exception('body must by dict')
 
@@ -55,12 +58,15 @@ class Body:
     @property
     def login(self):
         from isc_common.auth.models.user import User
-
         if self.body.login != None:
             return self.body.login
-        elif self.body.httpHeaders.USER_ID:
+        elif self.body.httpHeaders != None and self.body.httpHeaders.USER_ID:
             return User.objects.get(id=self.body.httpHeaders.USER_ID).username
         else:
+            if self.session != None and self.session._session != None:
+                username = self.session._session.get('ws_channel').split('_')[1]
+                return username
+
             raise Exception('user not enable type')
 
 def JsonWSResponseWithException(printing=False, printing_res=False):
@@ -109,7 +115,7 @@ def JsonWSResponseWithException(printing=False, printing_res=False):
                 self.channel = request.GET.get('ws_channel')
                 if self.channel == None:
                     body = BytesToJson(request.body)
-                    body = Body(body)
+                    body = Body(body, request.session)
                     if settings.WS_CHANNEL != None:
                         self.channel = f'''{settings.WS_CHANNEL}_{body.login}'''
 
@@ -242,7 +248,7 @@ def JsonWSPostResponseWithException(printing=False, printing_res=False):
 
             self.port = request.POST.get('ws_port')
             if self.port == None:
-                self.host = settings.WS_PORT
+                self.port = settings.WS_PORT
                 if not self.port:
                     raise Exception(Common.arraund_error('Do not have ws_port'))
 
@@ -255,7 +261,7 @@ def JsonWSPostResponseWithException(printing=False, printing_res=False):
             self.channel = request.POST.get('ws_channel')
             if self.channel == None:
                 body = BytesToJson(request.body)
-                body = Body(body)
+                body = Body(body, request.session)
                 if settings.WS_CHANNEL != None:
                     self.channel = f'''{settings.WS_CHANNEL}_{body.login}'''
                     if self.channel == None:
