@@ -32,7 +32,10 @@ class AuditQuerySet(CommonQuerySet):
 
     def soft_delete(self):
         res = super().update(deleted_at=timezone.now())
-        return res
+        if res > 0:
+            return self
+        else:
+            return None
 
     def soft_restore(self):
         res = super().update(deleted_at=None)
@@ -108,6 +111,10 @@ class AuditModel(Model):
     def idHidden(self):
         return not self == None
 
+    @property
+    def is_deleted(self):
+        return self.deleted_at != None
+
     class Meta:
         abstract = True
 
